@@ -2,7 +2,8 @@
  * gedit-document-saver.h
  * This file is part of gedit
  *
- * Copyright (C) 2005 - Paolo Maggi 
+ * Copyright (C) 2005 - Paolo Maggi
+ * Copyrhing (C) 2007 - Paolo Maggi, Steve Frécinaux
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,16 +17,16 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
+ * Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
- 
+
 /*
- * Modified by the gedit Team, 2005. See the AUTHORS file for a 
- * list of people on the gedit Team.  
- * See the ChangeLog files for a list of changes. 
+ * Modified by the gedit Team, 2005. See the AUTHORS file for a
+ * list of people on the gedit Team.
+ * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-document-saver.h 5666 2007-06-29 19:52:25Z sfre $
+ * $Id: gedit-document-saver.h 5756 2007-07-29 17:30:40Z sfre $
  */
 
 #ifndef __GEDIT_DOCUMENT_SAVER_H__
@@ -46,9 +47,6 @@ G_BEGIN_DECLS
 #define GEDIT_IS_DOCUMENT_SAVER_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GEDIT_TYPE_DOCUMENT_SAVER))
 #define GEDIT_DOCUMENT_SAVER_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS((obj), GEDIT_TYPE_DOCUMENT_SAVER, GeditDocumentSaverClass))
 
-/* Private structure type */
-typedef struct _GeditDocumentSaverPrivate GeditDocumentSaverPrivate;
-
 /*
  * Main object structure
  */
@@ -58,8 +56,18 @@ struct _GeditDocumentSaver
 {
 	GObject object;
 
-	/*< private > */
-	GeditDocumentSaverPrivate *priv;
+	/*< private >*/
+	GeditDocument		 *document;
+	gboolean		  used;
+
+	gchar			 *uri;
+	const GeditEncoding      *encoding;
+
+	GeditDocumentSaveFlags    flags;
+
+	gboolean		  keep_backup;
+	gchar			 *backup_ext;
+	gboolean                  backups_in_curr_dir;
 };
 
 /*
@@ -71,9 +79,18 @@ struct _GeditDocumentSaverClass
 {
 	GObjectClass parent_class;
 
+	/* Signals */
 	void (* saving) (GeditDocumentSaver *saver,
 			 gboolean             completed,
 			 const GError        *error);
+
+	/* VTable */
+	void			(* save)		(GeditDocumentSaver *saver,
+							 time_t              old_mtime);
+	const gchar *		(* get_mime_type)	(GeditDocumentSaver *saver);
+	time_t			(* get_mtime)		(GeditDocumentSaver *saver);
+	GnomeVFSFileSize	(* get_file_size)	(GeditDocumentSaver *saver);
+	GnomeVFSFileSize	(* get_bytes_written)	(GeditDocumentSaver *saver);
 };
 
 /*
@@ -81,15 +98,23 @@ struct _GeditDocumentSaverClass
  */
 GType 		 	 gedit_document_saver_get_type		(void) G_GNUC_CONST;
 
-GeditDocumentSaver 	*gedit_document_saver_new 		(GeditDocument        *doc);
-
 /* If enconding == NULL, the encoding will be autodetected */
-void			 gedit_document_saver_save		(GeditDocumentSaver  *saver,
-							 	 const gchar         *uri,
-								 const GeditEncoding *encoding,
-								 time_t               oldmtime,
+GeditDocumentSaver 	*gedit_document_saver_new 		(GeditDocument        *doc,
+								 const gchar          *uri,
+								 const GeditEncoding  *encoding,
 								 GeditDocumentSaveFlags flags);
 
+gboolean		 gedit_document_saver_write_document_contents (
+								 GeditDocumentSaver  *saver,
+								 gint                 fd,
+								 GError             **error);
+
+void			 gedit_document_saver_saving		(GeditDocumentSaver *saver,
+								 gboolean            completed,
+								 GError             *error);
+void			 gedit_document_saver_save		(GeditDocumentSaver  *saver,
+								 time_t               old_mtime);
+
 #if 0
 void			 gedit_document_saver_cancel		(GeditDocumentSaver  *saver);
 #endif
@@ -106,9 +131,9 @@ const gchar		*gedit_document_saver_get_mime_type	(GeditDocumentSaver  *saver);
 time_t			 gedit_document_saver_get_mtime		(GeditDocumentSaver  *saver);
 
 /* Returns 0 if file size is unknown */
-GnomeVFSFileSize	 gedit_document_saver_get_file_size	(GeditDocumentSaver  *saver);									 
+GnomeVFSFileSize	 gedit_document_saver_get_file_size	(GeditDocumentSaver  *saver);
 
-GnomeVFSFileSize	 gedit_document_saver_get_bytes_written	(GeditDocumentSaver  *saver);									 
+GnomeVFSFileSize	 gedit_document_saver_get_bytes_written	(GeditDocumentSaver  *saver);
 
 
 G_END_DECLS
