@@ -25,7 +25,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-file-chooser-dialog.c 5855 2007-08-27 18:13:10Z paolo $
+ * $Id: gedit-file-chooser-dialog.c 5963 2007-10-11 08:36:23Z pborelli $
  */
 
 /* TODO: Override set_extra_widget */
@@ -168,7 +168,7 @@ all_text_files_filter (const GtkFileFilterInfo *filter_info,
 		lm = gedit_get_language_manager ();
 		languages = gtk_source_language_manager_get_language_ids (lm);
 
-		while (*languages != NULL)
+		while ((languages != NULL) && (*languages != NULL))
 		{
 			gchar **mime_types;
 			gint i;
