@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-view.c 5744 2007-07-26 10:11:16Z jessevdk $ 
+ * $Id: gedit-view.c 6063 2008-01-08 17:37:20Z wwalker $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -68,8 +68,6 @@ enum
 
 struct _GeditViewPrivate
 {
-	GtkTooltips *tooltips;
-
 	/* idle hack to make open-at-line work */
 	guint        scroll_idle;
 	
@@ -307,10 +305,10 @@ gedit_view_init (GeditView *view)
 		      "wrap_mode", gedit_prefs_manager_get_wrap_mode (),
 		      "show_line_numbers", gedit_prefs_manager_get_display_line_numbers (),
 		      "auto_indent", gedit_prefs_manager_get_auto_indent (),
-		      "tabs_width", gedit_prefs_manager_get_tabs_size (),
+		      "tab_width", gedit_prefs_manager_get_tabs_size (),
 		      "insert_spaces_instead_of_tabs", gedit_prefs_manager_get_insert_spaces (),
-		      "show_margin", gedit_prefs_manager_get_display_right_margin (), 
-		      "margin", gedit_prefs_manager_get_right_margin_position (),
+		      "show_right_margin", gedit_prefs_manager_get_display_right_margin (), 
+		      "right_margin_position", gedit_prefs_manager_get_right_margin_position (),
 		      "highlight_current_line", gedit_prefs_manager_get_highlight_current_line (),
 		      "smart_home_end", gedit_prefs_manager_get_smart_home_end (),
 		      "indent_on_tab", TRUE,
@@ -372,9 +370,6 @@ gedit_view_finalize (GObject *object)
 	view = GEDIT_VIEW (object);
 
 	g_free (view->priv->old_search_text);
-	
-	if (view->priv->tooltips != NULL)
-		g_object_unref (view->priv->tooltips);
 
 	(* G_OBJECT_CLASS (gedit_view_parent_class)->finalize) (object);
 }
@@ -608,22 +603,28 @@ gedit_view_set_font (GeditView   *view,
 }
 
 static void
-add_search_completion_entry (const gchar *text)
+add_search_completion_entry (const gchar *str)
 {
+	gchar        *text;
 	gboolean      valid;
 	GtkTreeModel *model;
 	GtkTreeIter   iter;
-	
-	if (text == NULL)
+
+	if (str == NULL)
 		return;
-				
+
+	text = gedit_utils_unescape_search_text (str);
+
 	if (g_utf8_strlen (text, -1) < MIN_SEARCH_COMPLETION_KEY_LEN)
+	{
+		g_free (text);
 		return;
-	
+	}
+
 	g_return_if_fail (GTK_IS_TREE_MODEL (search_completion_model));
-	
+
 	model = GTK_TREE_MODEL (search_completion_model);	
-		
+
 	/* Get the first iter in the list */
 	valid = gtk_tree_model_get_iter_first (model, &iter);
 
@@ -631,7 +632,7 @@ add_search_completion_entry (const gchar *text)
 	{
 		/* Walk through the list, reading each row */
      		gchar *str_data;
-      
+
 		gtk_tree_model_get (model, 
 				    &iter, 
                           	    0, 
@@ -640,6 +641,7 @@ add_search_completion_entry (const gchar *text)
 
 		if (strcmp (text, str_data) == 0)
 		{
+			g_free (text);
 			g_free (str_data);
 			gtk_list_store_move_after (GTK_LIST_STORE (model),
 						   &iter,
@@ -652,14 +654,16 @@ add_search_completion_entry (const gchar *text)
 
 		valid = gtk_tree_model_iter_next (model, &iter);
     	}
-        
+
 	gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
-	gtk_list_store_set (GTK_LIST_STORE (model), 
-			    &iter, 
-			    0, 
-			    text, 
+	gtk_list_store_set (GTK_LIST_STORE (model),
+			    &iter,
+			    0,
+			    text,
 			    -1);
-}		
+
+	g_free (text);
+}
 
 static void
 set_entry_background (GtkWidget               *entry,
@@ -869,6 +873,9 @@ hide_search_window (GeditView *view, gboolean cancel)
 		
 		gedit_view_scroll_to_cursor (view);
 	}
+
+	/* make sure a focus event is sent for the edit area */
+	send_focus_change (GTK_WIDGET (view), TRUE);
 }
 
 static gboolean
@@ -1236,21 +1243,17 @@ customize_for_search_mode (GeditView *view)
 	{
 		icon = gtk_image_new_from_stock (GTK_STOCK_FIND,
 						 GTK_ICON_SIZE_MENU);
-		
-		gtk_tooltips_set_tip (view->priv->tooltips,
-				      view->priv->search_entry,
-				      _("String you want to search for"),
-				      NULL);
+
+		gtk_widget_set_tooltip_text (view->priv->search_entry,
+					     _("String you want to search for"));
 	}
 	else
 	{
 		icon = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO,
 						 GTK_ICON_SIZE_MENU);
-		
-		gtk_tooltips_set_tip (view->priv->tooltips,
-				      view->priv->search_entry,
-				      _("Line you want to move the cursor to"),
-				      NULL);
+
+		gtk_widget_set_tooltip_text (view->priv->search_entry,
+					     _("Line you want to move the cursor to"));
 	}
 
 	gtk_widget_show (icon);
@@ -1430,9 +1433,6 @@ ensure_search_window (GeditView *view)
 
 	gtk_widget_realize (view->priv->search_entry);
 
-	view->priv->tooltips = gtk_tooltips_new ();
-	g_object_ref_sink (view->priv->tooltips);
-
 	customize_for_search_mode (view);	
 }
 
