@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.c 5987 2007-10-28 10:28:46Z pborelli $
+ * $Id: gedit-window.c 6116 2008-01-24 22:15:19Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -120,7 +120,7 @@ gedit_window_dispose (GObject *object)
 	/* First of all, force collection so that plugins
 	 * really drop some of the references.
 	 */
-	gedit_plugins_engine_garbage_collect ();
+	gedit_plugins_engine_garbage_collect (gedit_plugins_engine_get_default ());
 
 	if (window->priv->recents_handler_id != 0)
 	{
@@ -147,7 +147,7 @@ gedit_window_dispose (GObject *object)
 	/* Now that there have broken some reference loops,
 	 * force collection again.
 	 */
-	gedit_plugins_engine_garbage_collect ();
+	gedit_plugins_engine_garbage_collect (gedit_plugins_engine_get_default ());
 
 	G_OBJECT_CLASS (gedit_window_parent_class)->dispose (object);
 }
@@ -286,7 +286,7 @@ static void
 gedit_window_tab_removed (GeditWindow *window,
 			  GeditTab    *tab) 
 {
-	gedit_plugins_engine_garbage_collect();
+	gedit_plugins_engine_garbage_collect (gedit_plugins_engine_get_default ());
 }
 
 static void
@@ -364,8 +364,9 @@ gedit_window_class_init (GeditWindowClass *klass)
 							     "The window's state",
 							     GEDIT_TYPE_WINDOW_STATE,
 							     GEDIT_WINDOW_STATE_NORMAL,
-							     G_PARAM_READABLE));
-							   
+							     G_PARAM_READABLE |
+							     G_PARAM_STATIC_STRINGS));
+
 	g_type_class_add_private (object_class, sizeof(GeditWindowPrivate));
 }
 
@@ -679,6 +680,8 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 	                          (state != GEDIT_TAB_STATE_CLOSING) &&
 				  (state != GEDIT_TAB_STATE_SAVING) &&
 				  (state != GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW) &&
+				  (state != GEDIT_TAB_STATE_PRINTING) &&
+				  (state != GEDIT_TAB_STATE_PRINT_PREVIEWING) &&
 				  (state != GEDIT_TAB_STATE_SAVING_ERROR));
 
 	action = gtk_action_group_get_action (window->priv->action_group,
@@ -764,7 +767,8 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 
 	update_next_prev_doc_sensitivity (window, tab);
 	
-	gedit_plugins_engine_update_plugins_ui (window, FALSE);
+	gedit_plugins_engine_update_plugins_ui (gedit_plugins_engine_get_default (),
+						window, FALSE);
 }
 
 static void
@@ -1109,7 +1113,7 @@ recents_menu_activate (GtkAction   *action,
 static gint
 sort_recents_mru (GtkRecentInfo *a, GtkRecentInfo *b)
 {
-	return (gtk_recent_info_get_modified (a) < gtk_recent_info_get_modified (b));
+	return (gtk_recent_info_get_modified (b) - gtk_recent_info_get_modified (a));
 }
 
 static void	update_recent_files_menu (GeditWindow *window);
@@ -2241,7 +2245,8 @@ sync_name (GeditTab    *tab,
 	g_free (escaped_name);
 	g_free (tip);
 
-	gedit_plugins_engine_update_plugins_ui (window, FALSE);
+	gedit_plugins_engine_update_plugins_ui (gedit_plugins_engine_get_default (),
+						window, FALSE);
 }
 
 static GeditWindow *
@@ -2437,7 +2442,8 @@ editable_changed (GeditView  *view,
                   GParamSpec  *arg1,
                   GeditWindow *window)
 {
-	gedit_plugins_engine_update_plugins_ui (window, FALSE);
+	gedit_plugins_engine_update_plugins_ui (gedit_plugins_engine_get_default (),
+						window, FALSE);
 }
 
 static void
@@ -2619,7 +2625,8 @@ notebook_tab_removed (GeditNotebook *notebook,
 						      "ViewHighlightMode");
 		gtk_action_set_sensitive (action, FALSE);
 
-		gedit_plugins_engine_update_plugins_ui (window, FALSE);
+		gedit_plugins_engine_update_plugins_ui (gedit_plugins_engine_get_default (),
+							window, FALSE);
 	}
 
 	if (window->priv->num_tabs <= 1)
@@ -3179,7 +3186,8 @@ gedit_window_init (GeditWindow *window)
 			  NULL);
 
 	gedit_debug_message (DEBUG_WINDOW, "Update plugins ui");
-	gedit_plugins_engine_update_plugins_ui (window, TRUE);
+	gedit_plugins_engine_update_plugins_ui (gedit_plugins_engine_get_default (),
+						window, TRUE);
 
 	/* set visibility of panes.
 	 * This needs to be done after plugins activatation */
